@@ -1,7 +1,7 @@
 package Time::Duration::Parse::AsHash;
 
-our $DATE = '2015-09-26'; # DATE
-our $VERSION = '0.10.2'; # VERSION
+our $DATE = '2016-05-17'; # DATE
+our $VERSION = '0.10.3'; # VERSION
 
 #IFUNBUILT
 use strict;
@@ -12,13 +12,24 @@ use Exporter;
 our @ISA    = qw( Exporter );
 our @EXPORT = qw( parse_duration );
 
-my %Units = ( map(($_, "seconds"), qw(s second seconds sec secs)),
+my %Units = ( map(($_, "nanoseconds" ), qw(ns nanosecond nanoseconds)),
+              map(($_, "milliseconds"), qw(ms millisecond milliseconds milisecond miliseconds)),
+              map(($_, "microseconds"), qw(microsecond microseconds)),
+              map(($_, "seconds"), qw(s second seconds sec secs)),
               map(($_, "minutes"), qw(m minute minutes min mins)),
               map(($_,   "hours"), qw(h hr hour hours)),
               map(($_,    "days"), qw(d day days)),
               map(($_,   "weeks"), qw(w week weeks)),
               map(($_,  "months"), qw(M month months mon mons mo mos)),
-              map(($_,   "years"), qw(y year years)) );
+              map(($_,   "years"), qw(y year years)),
+              map(($_, "decades"), qw(decade decades)),
+          );
+my %Converts = (
+    nanoseconds  => ["seconds" => 1e-9],
+    microseconds => ["seconds" => 1e-6],
+    milliseconds => ["seconds" => 1e-3],
+    decades      => ["years"   => 10],
+);
 
 sub parse_duration {
     my $timespec = shift;
@@ -32,7 +43,7 @@ sub parse_duration {
     }
 
     # Convert hh:mm(:ss)? to something we understand
-    $timespec =~ s/\b(\d+):(\d\d):(\d\d)\b/$1h $2m $3s/g;
+    $timespec =~ s/\b(\d+):(\d\d):(\d\d(?:\.\d+)?)\b/$1h $2m $3s/g;
     $timespec =~ s/\b(\d+):(\d\d)\b/$1h $2m/g;
 
     my %res;
@@ -42,6 +53,10 @@ sub parse_duration {
 
         if (my $canon_unit = $Units{$unit}) {
             $amount =~ s/,/./;
+            if (my $convert = $Converts{$canon_unit}) {
+                $canon_unit = $convert->[0];
+                $amount *= $convert->[1];
+            }
             $res{$canon_unit} += $amount;
         } else {
             die "Unknown timespec: $1 $2";
@@ -74,7 +89,7 @@ Time::Duration::Parse::AsHash - Parse string that represents time duration
 
 =head1 VERSION
 
-version 0.10.2
+version 0.10.3
 
 =head1 SYNOPSIS
 
@@ -84,25 +99,40 @@ version 0.10.2
 
 =head1 DESCRIPTION
 
-Time::Duration::Parse::AsHash is like L<Time::Duration::Parse> but it returns
-parsed period elements instead of number of seconds. There are some
-circumstances when you want this, e.g. when feeding into L<DateTime::Duration>
-and you want to count for leap seconds.
+Time::Duration::Parse::AsHash is like L<Time::Duration::Parse> except:
 
-Also, unlike Time::Duration::Parse, seconds are not rounded by default.
+=over
+
+=item * It returns a hashref of parsed duration elements instead of number of seconds
+
+There are some circumstances when you want this, e.g. when feeding into
+L<DateTime::Duration> and you want to count for leap seconds.
+
+=item * Seconds are not rounded by default
+
+For example: C<"0.1s"> or C<100ms> will return result C<< { seconds => 0.1 } >>.
+
+Also, in addition to C<01:02:03> being recognized as C<1h2min3s>,
+C<01:02:03.4567> will also be recognized as C<1h2min3.4567s>.
+
+=item * Extra elements recognized
+
+C<milliseconds> (or C<ms>). This will be returned in C<seconds> key.
+
+C<microseconds>. This will also be returned in C<seconds> key.
+
+C<nanoseconds> (or C<ns>). This will also be returned in C<seconds> key.
+
+C<decades>. This will be returned in C<years> key.
+
+=back
 
 =head1 FUNCTIONS
 
-=over 4
-
-=item parse_duration
-
-  $res = parse_duration($string);
+=head2 parse_duration(str) => hash
 
 Parses duration string and returns hash. This function is exported by default.
 
-=back
-
 =head1 SEE ALSO
 
 L<Time::Duration::Parse>
@@ -113,7 +143,7 @@ perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2015 by perlancar@cpan.org.
+This software is copyright (c) 2016 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
